
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <GL/gl.h>

#include "defines.h"
#include "png.h"



int main(int argc, char *argv[]) {

  unsigned char *d, *t, m[4];
  int dx, dy, type, i, j, n, x, y;
  FILE *f;


  if (argc != 3) {
    fprintf(stderr, "USAGE: %s <PNG FILE> <OUT FILE>\n", argv[0]);
    return -1;
  }

  if (png_load(argv[1], &dx, &dy, &type, &d) == FAILED)
    return -1;

  /* no alpha? no saving? */
  if (type == GL_RGB)
    n = 3;
  else
    n = 4;

  t = malloc(20*(18*4));
  if (t == NULL) {
    fprintf(stderr, "MAIN: Out of memory error.\n");
    return -1;
  }

  for (i = 0, y = 0; y < 18*4; y++) {
    for (x = 0; x < 20; x++, i++) {
      m[0] = d[y*dx*n + x*4*n + 0*n];
      m[1] = d[y*dx*n + x*4*n + 1*n];
      m[2] = d[y*dx*n + x*4*n + 2*n];
      m[3] = d[y*dx*n + x*4*n + 3*n];
      t[i] = ((m[0]>>6)<<6) | ((m[1]>>6)<<4) | ((m[2]>>6)<<2) | (m[3]>>6);
    }
  }

  f = fopen(argv[2], "wb");
  if (f == NULL) {
    fprintf(stderr, "MAIN: Could not open file \"%s\" for writing.\n", argv[2]);
    free(t);
    free(d);
    return 1;
  }

  /* the first scanline is pure data */
  fprintf(f, "; the first row of the image \"%s\"\n", argv[1]);
  fprintf(f, ".db ");
  for (i = 0; i < 19; i++)
    fprintf(f, "%d, ", t[i]);
  fprintf(f, "%d\n", t[i++]);

  /* write the hblank interrupts */
  n = 0x9800;
  for (j = 1, y = 0; y < 18*4-1; y++, j++) {
    if (j == 4) {
      j = 0;
      n += 32;
    }

    fprintf(f, "; hblank irq\n");

    fprintf(f, "       push    af\n");
    fprintf(f, "       push    hl\n");
    fprintf(f, "       ld      hl, %d\n", n);

    /* 5 BYTES so far... */

    for (x = 0; x < 19; x++) {
      fprintf(f, "       ld      a, %d\n", t[i]);
      i++;
      fprintf(f, "       ld      (hl+), a\n");
    }
    fprintf(f, "       ld      (hl), %d\n", t[i]);
    i++;

    /* 5+59 BYTES so far... */

    fprintf(f, "       ld      hl, IRQ_LCD+1\n");
    fprintf(f, "       ld      a, SM4x4_SKIP&$ff  ; low.\n");
    fprintf(f, "       ld      (hl+), a\n");
    fprintf(f, "       ld      a, SM4x4_SKIP>>8   ; high.\n");
    fprintf(f, "       ld      (hl), a\n");

    fprintf(f, "       pop     hl\n");
    fprintf(f, "       pop     af\n");
    fprintf(f, "       reti\n");

    /* 5+59+12=76 BYTES */
  }
  fprintf(f, "; the last hblank irq\n");
  fprintf(f, "       reti\n");

  fclose(f);

  free(t);
  free(d);

  return 0;
}
